
from flask_mail import Message as FlaskMessage, sanitize_subject, force_text, sanitize_address, sanitize_addresses
from flask import current_app
from email.encoders import encode_base64
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage
from email.message import EmailMessage
from email.utils import formatdate, formataddr, make_msgid, parseaddr
import re
import os
import requests
import mimetypes
import unicodedata
import sys

PY3 = sys.version_info[0] == 3

if PY3:
    from email import policy
    message_policy = policy.SMTP
else:
    message_policy = None


class Message(FlaskMessage):
    _embed_images = False

    def _message(self):
        """Creates the email"""
        ascii_attachments = current_app.extensions['mail'].ascii_attachments
        encoding = self.charset or 'utf-8'

        attachments = self.attachments or []

        embed_images = []
        if self._embed_images and self.html:
            for img_url in re.findall(r'<img.+?src=[\"\'](.+?)[\"\'].*?>', self.html):
                if not img_url.startswith('cid:'):
                    img_msg_id = make_msgid()
                    embed_images.append((img_url, img_msg_id))
                    self.html = self.html.replace(img_url, 'cid:{}'.format(img_msg_id[1:-1]))

        msg = EmailMessage()

        if self.body and self.html:
            html_index = 1
            msg.set_content(self.body)
            msg.add_alternative(self.html, subtype='html')
        elif self.body:
            html_index = 0
            msg.set_content(self.body)
        else:
            html_index = 1
            msg.set_content(self.html)
            msg.add_alternative(self.html, subtype='html')

        if self.subject:
            msg['Subject'] = force_text(self.subject)

        msg['From'] = sanitize_address(self.sender, encoding)
        msg['To'] = ', '.join(list(set(sanitize_addresses(self.recipients, encoding))))

        msg['Date'] = formatdate(self.date, localtime=True)
        # see RFC 5322 section 3.6.4.
        msg['Message-ID'] = self.msgId

        if self.cc:
            msg['Cc'] = ', '.join(list(set(sanitize_addresses(self.cc, encoding))))

        if self.reply_to:
            msg['Reply-To'] = sanitize_address(self.reply_to, encoding)

        if self.extra_headers:
            for k, v in self.extra_headers.items():
                msg[k] = v

        SPACES = re.compile(r'[\s]+', re.UNICODE)
        for attachment in attachments:
            f = MIMEBase(*attachment.content_type.split('/'))
            f.set_payload(attachment.data)
            encode_base64(f)

            filename = attachment.filename
            if filename and ascii_attachments:
                # force filename to ascii
                filename = unicodedata.normalize('NFKD', filename)
                filename = filename.encode('ascii', 'ignore').decode('ascii')
                filename = SPACES.sub(u' ', filename).strip()

            try:
                filename and filename.encode('ascii')
            except UnicodeEncodeError:
                if not PY3:
                    filename = filename.encode('utf8')
                filename = ('UTF8', '', filename)

            f.add_header('Content-Disposition',
                         attachment.disposition,
                         filename=filename)

            for key, value in attachment.headers:
                f.add_header(key, value)

            msg.attach(f)

        for (img_url, uuid) in embed_images:
            basename = os.path.basename(img_url)
            maintype, subtype = mimetypes.guess_type(str(basename))[0].split('/', 1)
            res = requests.get(img_url)
            msg.get_payload(html_index).add_related(  # image/png
                res.content, maintype, subtype, cid=uuid)

        if message_policy:
            msg.policy = message_policy

        return msg

    def embed_images(self):
        self._embed_images = True
